@@ -34,6 +34,7 @@ class Database:
 
             
             if price_adj == False:
+                print(response.content)
 
                 ohlc_result = json.loads(json.loads(response.content).replace("'", "\"").replace('False','0').replace('True','1'))
             else:
@@ -217,6 +218,7 @@ class Database:
                 if year_month not in year_month_involved_list:
                     year_month_involved_list.append(year_month)
 
+            #if end_date >= int(expiry_date_dict[max(year_month_involved_list)].strftime('%Y%m%d')):
             year_month_involved_list.append((datetime.datetime.strptime(year_month_involved_list[-1], '%y%m') + relativedelta(months=1)).strftime('%y%m'))
 
             year_month_involved_list_str = json.dumps(str(year_month_involved_list))
@@ -246,6 +248,7 @@ class Database:
             result_list = json.loads(json.loads(response.content).replace("'", "\""))
 
             df_list = []
+            date_time_list_list = []
 
             for result in result_list:
                 sub_df_list = []
@@ -272,7 +275,7 @@ class Database:
 
                     for i in range(len(date_list)):
                         date_item = date_list[i]
-    
+
                         if i + rolling_day <= len(date_list) - 1:
                             if date_list[i + rolling_day] == front_expiry_date:
                                 front_cut_off_date = date_item
@@ -289,24 +292,35 @@ class Database:
                         df = df[(df['date'] > front_cut_off_date) | ((df['date'] == front_cut_off_date) & (df['time'] >= rolling_time))]
                         df = df[(df['date'] < back_cut_off_date) | ((df['date'] == back_cut_off_date) & (df['time'] < rolling_time))]
 
-                    df_list.append(df)
+                    df = df[df['date'] >= start_date]
+                    df = df[df['date'] <= end_date]
+                    df['date'] = df['date'].astype(str)
+                    df['time'] = df['time'].astype(str)
+                    df['time'] = df['time'].str.zfill(6)
+                    df['datetime'] = df['date'] + ' ' + df['time']
+
+                    df = df.reset_index(drop=True)
+                    date_time_list = df['datetime'].to_list()
+
+                    if len(date_time_list_list) > 0:
+                        date_time_intersect = (set(date_time_list_list).intersection(date_time_list))
+                        if len(date_time_intersect) == 0:
+                            df_list.append(df)
+                    else:
+                        df_list.append(df)
+
+                    date_time_list_list += date_time_list
 
             cols = ['datetime', 'date', 'time', 'open', 'high', 'low', 'close', 'volume', 'expiry_date','RTH']
             df = pd.concat(df_list)
 
-            df = df[df['date'] >= start_date]
-            df = df[df['date'] <= end_date]
-            df['date'] = df['date'].astype(str)
-            df['time'] = df['time'].astype(str)
-            df['time'] = df['time'].str.zfill(6)
-            df['datetime'] = df['date'] + ' ' + df['time']
             df['datetime'] = pd.to_datetime(df['datetime'], format='%Y%m%d %H%M%S')
             df['RTH'] = df['RTH'].astype(bool)
             
             df = df[cols]
             df = df.set_index(keys='datetime')
             df = df.sort_index(ascending=True)
-    
+
             return df
     
         else:
@@ -914,8 +928,8 @@ class Database:
         link_str = link_url + '&' + token_str + '&' + database_str
         
         response = requests.get(link_str)
-        #result = json.loads(json.loads(response.content).replace("'", "\""))
-        result = ast.literal_eval(json.loads(response.content))
+        json1 = json.loads(response.content).replace("'", "\"").replace('nan','NaN')
+        result = json.loads(json1)
 
         df = pd.DataFrame(result)
         df['ipo_date'] = pd.to_datetime(df['ipo_date'],format='%Y-%m-%d')
@@ -929,8 +943,7 @@ class Database:
         
         df['share_issued'] = df['share_issued'].astype(int)
         df['market_capital'] = df['market_capital'].astype(int)
-        
-        
+
         return df
 
     def get_hk_ipo_hist(self):
